window.Gleam.initSearch([{"doc":"category_theory","title":"category_theory","content":"# category_theory\n\n[![Package Version](https://img.shields.io/hexpm/v/category_theory)](https://hex.pm/packages/category_theory)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/category_theory/)\n\nThis package implements several category theory concepts, following [this book](https://bartoszmilewski.com/2014/10/28/category-theory-for-programmers-the-preface/) by Bartosz Milewski.\n\n```sh\ngleam add category_theory@1\n```\n\n```gleam\nimport category_theory.{Either, Left, Right}\nimport category_theory/monoid as mono\n\npub fn main() {\n   let either_sum_monoid =\n    mono.Monoid(\n      mempty: Left(0),\n      mappend: fn(e1: Either(Int, String), e2: Either(Int, String)) -&gt; Either(Int, String) {\n        case e1, e2 {\n          Right(s), _ -&gt; Right(s)\n          _, Right(s) -&gt; Right(s)\n          Left(a), Left(b) -&gt; Left(a + b)\n        }\n      }\n    )\n\n  either_sum_monoid\n  |&gt; mono.mconcat([Left(2), Left(3), Left(4)])\n  |&gt; io.debug()\n  // -&gt; Left(9)\n\n  either_sum_monoid\n  |&gt; mono.mconcat([Left(2), Right(&quot;error&quot;), Left(4)])\n  |&gt; io.debug()\n  // -&gt; Right(&quot;error&quot;)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/category_theory&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"category_theory","title":"Either","content":"pub type Either(a, b) {\n  Left(a)\n  Right(b)\n}\n Canonical implementation of a `coproduct` (sum type).\n \n data Either a b = Left a | Right b\n \n ### Examples\n \n let check_positive = fn(x: Int) -&gt; Either(Int, String) {\n   case x &gt;= 0 {\n     True -&gt; Left(x)\n     False -&gt; Right(&quot;negative number&quot;)\n   }\n }\n check_positive(12)\n // -&gt; Left(12)\n check_positive(-3)\n // -&gt; Right(&quot;negative number&quot;)\n \nLeft(a)\n\n\nRight(b)\n\n\nSynonyms:\ncategory_theory.Either\ncategory_theory Either","url":"category_theory.html#Either"},{"doc":"category_theory","title":"List","content":"pub type List(a) {\n  Null\n  Cons(List(a))\n}\n\nNull\n\n\nCons(List(a))\n\n\nSynonyms:\ncategory_theory.List\ncategory_theory List","url":"category_theory.html#List"},{"doc":"category_theory","title":"Maybe","content":"pub type Maybe(a) {\n  Nothing\n  Just(a)\n}\n `Maybe` type from Haskell (`Option` in gleam).\n \n data Maybe = Nothing | Just a\n -- Equivalent: Sum type between `unit` and `a`\n type Maybe = Either () a\n \n ### Examples\n \n let safe_div = fn(a, b) {\n   case b != 0.0 {\n     True -&gt; Just(a /. b)\n     False -&gt; Nothing\n   }\n }\n safe_div(3.0, 4.0)\n // -&gt; Just(0,75)\n safe_div(3.0, 0.0)\n // -&gt; Nothing\n \nNothing\n\n\nJust(a)\n\n\nSynonyms:\ncategory_theory.Maybe\ncategory_theory Maybe","url":"category_theory.html#Maybe"},{"doc":"category_theory","title":"Pair","content":"pub type Pair(a, b) {\n  Pair(fst: a, snd: b)\n}\n Canonical implementation of a `product` (tuple).\n Examples\n \n let even_to_string = fn(x: Int) -&gt; Pair(String, Bool) {\n   Pair(int.to_string(x), x % 2 == 0)\n }\n even_to_string(82).fst\n // -&gt; &quot;82&quot;\n even_to_string(82).snd\n // -&gt; True\n even_to_string(83).snd\n // -&gt; False\n \nPair(fst: a, snd: b)\n\n\nSynonyms:\ncategory_theory.Pair\ncategory_theory Pair","url":"category_theory.html#Pair"},{"doc":"category_theory","title":"Void","content":"pub type Void {\n  Void(Void)\n}\n A type corresponding to an `empty set`. It is `not inhabited` by any values.\nVoid(Void)\n\n\nSynonyms:\ncategory_theory.Void\ncategory_theory Void","url":"category_theory.html#Void"},{"doc":"category_theory","title":"absurd","content":"pub fn absurd(: Void) -&gt; a\n A function that can `never` be called. It is polymorphic in the return type. \nSynonyms:\ncategory_theory.absurd\ncategory_theory absurd","url":"category_theory.html#absurd"},{"doc":"category_theory","title":"compose","content":"pub fn compose(g: fn(a) -&gt; b, f: fn(c) -&gt; a) -&gt; fn(c) -&gt; b\n Given a function `f` that takes an argument of type A and returns a B, and another function `g` that takes a B and returns a C, you can `compose` them by `passing the result of f to g`. \n \n (.) :: (b -&gt; c) -&gt; (a -&gt; b) -&gt; (a -&gt; c)\n (g . f) x = f (g x)\n \n Properties of composition:\n - Associativity h . (g . f) == (h . g) . f == h . g . f\n - Identity see [`id`](#id) for more info\n ### Examples\n \n let f = fn(x: Int) { int.to_string(x) }\n let g = fn(s: String) { s == &quot;28&quot; }\n let h = compose(g, f)\n // -&gt; h takes an int, transforms it into a string, then compares it to &quot;28&quot; and returns a bool\n  \nSynonyms:\ncategory_theory.compose\ncategory_theory compose","url":"category_theory.html#compose"},{"doc":"category_theory","title":"coproduct_factorizer","content":"pub fn coproduct_factorizer(\n  i: fn(a) -&gt; b,\n  j: fn(c) -&gt; b,\n) -&gt; fn(Either(a, c)) -&gt; b\n Produces the factorizing function from a `candidate` c with `two injections` i and j to the best coproduct (either). \\\n Property: i and j can be `reconstructed` from the canonical coproduct e  \\\n With m = coproduct_factorizer(i, j), we have:\n - i(x) = m(Left(x))\n - j(x) = m(Right(x))\n ### Examples\n \n // Given the candidate #(Int, Bool) with two injections from Int and Bool\n let i = fn(x: Int) {#(x, False)}\n let j = fn(x: Bool) {#(9, x)}\n // We show that Either(Int, Bool) is a better coproduct by finding the mapping m:\n let m = coproduct_factorizer(i, j)\n m(Left(2))\n // -&gt; #(2, True)  \n m(Right(False))\n // -&gt; #(9, False)  \n \nSynonyms:\ncategory_theory.coproduct_factorizer\ncategory_theory coproduct_factorizer","url":"category_theory.html#coproduct_factorizer"},{"doc":"category_theory","title":"id","content":"pub fn id(x: a) -&gt; a\n The `identity function` is a `unit of composition`.\n \n id :: a -&gt; a\n id a = a\n \n It follows the identity conditions:\n - f . id == f\n - id . f == f\n ### Examples\n \n id(3)\n // -&gt; 3\n id(&quot;abc&quot;)\n // -&gt; &quot;abc&quot;\n \nSynonyms:\ncategory_theory.id\ncategory_theory id","url":"category_theory.html#id"},{"doc":"category_theory","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\ncategory_theory.main\ncategory_theory main","url":"category_theory.html#main"},{"doc":"category_theory","title":"maybe_compose","content":"pub fn maybe_compose(\n  m1: fn(a) -&gt; Maybe(b),\n  m2: fn(b) -&gt; Maybe(c),\n) -&gt; fn(a) -&gt; Maybe(c)\n `Composition` for the Maybe type \n ### Examples\n \n let safe_reciprocal = fn(x) {\n   case x != 0.0 {\n     True -&gt; Just(1.0 /. x)\n     False -&gt; Nothing\n   }\n }\n let safe_root = fn(x) {\n   case x &gt;=. 0.0 {\n     True -&gt; Just(x |&gt; float.square_root() |&gt; result.unwrap(0.0))\n     False -&gt; Nothing\n   }\n }\n let safe_reciprocal_root = maybe_compose(safe_reciprocal, safe_root)\n // -&gt; a function that calculates sqrt(1/x)\n safe_reciprocal_root(0.25)\n // -&gt; Just(2.0)\n safe_reciprocal_root(0.0)\n // -&gt; Nothing\n safe_reciprocal_root(-2.0)\n // -&gt; Nothing\n \nSynonyms:\ncategory_theory.maybe_compose\ncategory_theory maybe_compose","url":"category_theory.html#maybe_compose"},{"doc":"category_theory","title":"maybe_id","content":"pub fn maybe_id(x: a) -&gt; Maybe(a)\n The `idenitity morphism` for the Maybe type.\n ### Examples\n \n maybe_id(25)\n // -&gt; Just(25)\n maybe_id(Nothing)\n // -&gt; Just(Nothing)\n \nSynonyms:\ncategory_theory.maybe_id\ncategory_theory maybe_id","url":"category_theory.html#maybe_id"},{"doc":"category_theory","title":"maybe_to_option","content":"pub fn maybe_to_option(m: Maybe(a)) -&gt; Option(a)\n Converts from `Maybe` to gleam `Option`.\n ### Examples\n \n maybe_to_option(Nothing)\n // -&gt; None\n maybe_to_option(Just(2))\n // -&gt; Some(2)\n \nSynonyms:\ncategory_theory.maybe_to_option\ncategory_theory maybe_to_option","url":"category_theory.html#maybe_to_option"},{"doc":"category_theory","title":"option_to_maybe","content":"pub fn option_to_maybe(o: Option(a)) -&gt; Maybe(a)\n Converts from gleam `Option` to `Maybe`.\n ### Examples\n \n option_to_maybe(None)\n // -&gt; Nothing\n option_to_maybe(Some(2))\n // -&gt; Just(2)\n \nSynonyms:\ncategory_theory.option_to_maybe\ncategory_theory option_to_maybe","url":"category_theory.html#option_to_maybe"},{"doc":"category_theory","title":"pair_to_tuple","content":"pub fn pair_to_tuple(p: Pair(a, b)) -&gt; #(a, b)\n Converts from `Pair` to gleam `Tuple`.\n ### Examples\n \n pair_to_tuple(Pair(2, True))\n // -&gt; #(2, True)\n \nSynonyms:\ncategory_theory.pair_to_tuple\ncategory_theory pair_to_tuple","url":"category_theory.html#pair_to_tuple"},{"doc":"category_theory","title":"product_factorizer","content":"pub fn product_factorizer(\n  p: fn(a) -&gt; b,\n  q: fn(a) -&gt; c,\n) -&gt; fn(a) -&gt; Pair(b, c)\n Produces the factorizing function from a `candidate` c with `two projections` p and q to the best product (tuple / pair). \\\n Property: p and q can be `reconstructed` from the canonical product  \\\n With m = product_factorizer(p, q), we have:\n - p(x) = m(x).fst\n - q(x) = m(x).snd\n ### Examples\n \n // Given the candidate Int with two projections to Int and Bool\n let p = fn(x: Int) {x}\n let q = fn(_: Int) {True}\n // We show that Pair(Int, Bool) is a better product by finding the mapping m:\n let m = product_factorizer(p, q)\n m(7)\n // -&gt; Pair(7, True)  \n \nSynonyms:\ncategory_theory.product_factorizer\ncategory_theory product_factorizer","url":"category_theory.html#product_factorizer"},{"doc":"category_theory","title":"tuple_to_pair","content":"pub fn tuple_to_pair(t: #(a, b)) -&gt; Pair(a, b)\n Converts from gleam `Tuple` to `Pair`.\n ### Examples\n \n tuple_to_pair(#(2, True))\n // -&gt; Pair(2, True)\n \nSynonyms:\ncategory_theory.tuple_to_pair\ncategory_theory tuple_to_pair","url":"category_theory.html#tuple_to_pair"},{"doc":"category_theory","title":"unit","content":"pub fn unit(: a) -&gt; Nil\n A function from any type to a unit (Nil in gleam).\n \n unit :: a -&gt; ()\n unit _ = ()\n \n ### Examples\n \n unit(42)\n // -&gt; Nil\n unit(True)\n // -&gt; Nil\n \nSynonyms:\ncategory_theory.unit\ncategory_theory unit","url":"category_theory.html#unit"},{"doc":"category_theory","title":"category_theory","content":" Module contains basic category concepts such as: composition, identity, unit, maybe, product, coproduct.","url":"category_theory.html"},{"doc":"category_theory/isomorphisms","title":"alpha","content":"pub fn alpha(p: Pair(a, Pair(b, c))) -&gt; Pair(Pair(a, b), c)\n Associativity `up to isomorphism` of nested Pairs. \\\n __a * (b * c) = (a * b) * c__\nSynonyms:\ncategory_theory/isomorphisms.alpha\ncategory_theory/isomorphisms alpha","url":"category_theory/isomorphisms.html#alpha"},{"doc":"category_theory/isomorphisms","title":"alpha_inv","content":"pub fn alpha_inv(p: Pair(Pair(a, b), c)) -&gt; Pair(a, Pair(b, c))\n Inverse associativity `up to isomorphism` of nested Pairs. \\\n __(a * b) * c = a * (b * c)__\nSynonyms:\ncategory_theory/isomorphisms.alpha_inv\ncategory_theory/isomorphisms alpha_inv","url":"category_theory/isomorphisms.html#alpha_inv"},{"doc":"category_theory/isomorphisms","title":"beta","content":"pub fn beta(\n  e1: Either(a, Either(b, c)),\n) -&gt; Either(Either(a, b), c)\n Associativity `up to isomorphism` of nested Eithers. \\\n __a + (b + c) = (a + b) + c__\nSynonyms:\ncategory_theory/isomorphisms.beta\ncategory_theory/isomorphisms beta","url":"category_theory/isomorphisms.html#beta"},{"doc":"category_theory/isomorphisms","title":"beta_inv","content":"pub fn beta_inv(\n  e1: Either(Either(a, b), c),\n) -&gt; Either(a, Either(b, c))\n Inverse associativity `up to isomorphism` of nested Eithers. \\\n __(a + b) + c = a + (b + c)__\nSynonyms:\ncategory_theory/isomorphisms.beta_inv\ncategory_theory/isomorphisms beta_inv","url":"category_theory/isomorphisms.html#beta_inv"},{"doc":"category_theory/isomorphisms","title":"delta","content":"pub fn delta(e: Either(a, a)) -&gt; Pair(Bool, a)\n Morphism from `Either(a, a)` to `Pair(Bool, a)`. \\\n __a + a = 2 * a__\nSynonyms:\ncategory_theory/isomorphisms.delta\ncategory_theory/isomorphisms delta","url":"category_theory/isomorphisms.html#delta"},{"doc":"category_theory/isomorphisms","title":"delta_inv","content":"pub fn delta_inv(p: Pair(Bool, a)) -&gt; Either(a, a)\n Morphism from `Pair(Bool, a)` to `Either(a, a)`. \\\n __2 * a = a + a__\nSynonyms:\ncategory_theory/isomorphisms.delta_inv\ncategory_theory/isomorphisms delta_inv","url":"category_theory/isomorphisms.html#delta_inv"},{"doc":"category_theory/isomorphisms","title":"omega","content":"pub fn omega(m: Maybe(a)) -&gt; Either(Nil, a)\n Morphism from `Maybe(a)` to `Either(Nil, a)`. \\\n __1 + a__\nSynonyms:\ncategory_theory/isomorphisms.omega\ncategory_theory/isomorphisms omega","url":"category_theory/isomorphisms.html#omega"},{"doc":"category_theory/isomorphisms","title":"omega_inv","content":"pub fn omega_inv(e: Either(Nil, a)) -&gt; Maybe(a)\n Inverse morphism from `Either(Nil, a)` to `Maybe(a)`. \\\n __1 + a__\nSynonyms:\ncategory_theory/isomorphisms.omega_inv\ncategory_theory/isomorphisms omega_inv","url":"category_theory/isomorphisms.html#omega_inv"},{"doc":"category_theory/isomorphisms","title":"product_to_sum","content":"pub fn product_to_sum(\n  p: Pair(a, Either(b, c)),\n) -&gt; Either(Pair(a, b), Pair(a, c))\n Distributivity `up to isomorphism`. \\\n __a * (b + c) = a * b + a * c__\nSynonyms:\ncategory_theory/isomorphisms.product_to_sum\ncategory_theory/isomorphisms product_to_sum","url":"category_theory/isomorphisms.html#product_to_sum"},{"doc":"category_theory/isomorphisms","title":"psi","content":"pub fn psi(e: Either(a, Void)) -&gt; a\n Morphism from `Either(a, Void)` to `a`. \\\n __a + 0 = a__\nSynonyms:\ncategory_theory/isomorphisms.psi\ncategory_theory/isomorphisms psi","url":"category_theory/isomorphisms.html#psi"},{"doc":"category_theory/isomorphisms","title":"psi_inv","content":"pub fn psi_inv(x: a) -&gt; Either(a, Void)\n Inverse morphism from `a` to `Either(a, Void)`. \\\n __a = a + 0__\nSynonyms:\ncategory_theory/isomorphisms.psi_inv\ncategory_theory/isomorphisms psi_inv","url":"category_theory/isomorphisms.html#psi_inv"},{"doc":"category_theory/isomorphisms","title":"rho","content":"pub fn rho(p: Pair(a, Nil)) -&gt; a\n Morphism from `(a, ())` to `a`. \\\n __a * 1 = a__\nSynonyms:\ncategory_theory/isomorphisms.rho\ncategory_theory/isomorphisms rho","url":"category_theory/isomorphisms.html#rho"},{"doc":"category_theory/isomorphisms","title":"rho_inv","content":"pub fn rho_inv(x: a) -&gt; Pair(a, Nil)\n Inverse morphism from `a` to `(a, ())`. \\\n __a = a * 1__\nSynonyms:\ncategory_theory/isomorphisms.rho_inv\ncategory_theory/isomorphisms rho_inv","url":"category_theory/isomorphisms.html#rho_inv"},{"doc":"category_theory/isomorphisms","title":"sum_to_product","content":"pub fn sum_to_product(\n  e: Either(Pair(a, b), Pair(a, c)),\n) -&gt; Pair(a, Either(b, c))\n Inverse distributivity `up to isomorphism`. \\\n __a * b + a * c = a * (b + c)__\nSynonyms:\ncategory_theory/isomorphisms.sum_to_product\ncategory_theory/isomorphisms sum_to_product","url":"category_theory/isomorphisms.html#sum_to_product"},{"doc":"category_theory/isomorphisms","title":"swap","content":"pub fn swap(p: Pair(a, b)) -&gt; Pair(b, a)\n Commutativity `up to isomorphism` for Pair. \\\n __a * b = b * a__\n ### Examples\n \n swap(Pair(5, &quot;abc&quot;))\n // -&gt; Pair(&quot;abc&quot;, 5)\n \nSynonyms:\ncategory_theory/isomorphisms.swap\ncategory_theory/isomorphisms swap","url":"category_theory/isomorphisms.html#swap"},{"doc":"category_theory/isomorphisms","title":"switch","content":"pub fn switch(e: Either(a, b)) -&gt; Either(b, a)\n Commutativity `up to isomorphism` for Either. \\\n __a + b = b + a__\n ### Examples\n \n swap(Left(5))\n // -&gt; Right(5)\n swap(Right(&quot;abc&quot;))\n // -&gt; Left(&quot;abc&quot;)\n \nSynonyms:\ncategory_theory/isomorphisms.switch\ncategory_theory/isomorphisms switch","url":"category_theory/isomorphisms.html#switch"},{"doc":"category_theory/isomorphisms","title":"category_theory/isomorphisms","content":" This module contains various isomorphisms (morphisms that are invertible).\n Algebra on Types (Void = 0, Nil = 1, Bool = 2, Either(a, b) = a + b, Pair(a, b) = a * b, Maybe(a) = 1 + a)","url":"category_theory/isomorphisms.html"},{"doc":"category_theory/monad","title":"Writer","content":"pub type Writer(a) {\n  Writer(a, String)\n}\n Encapsulates a pair whose first component is a `value` of arbitrary type a and the second component is a `string`. \\\n Used to `embellish` the return values of functions.\n ### Examples\n \n // Original function\n f = fn(x) {x * 2}\n // Embellished function\n f = fn(x) {Writer(x * 2, &quot;doubled &quot;)}\n \nWriter(a, String)\n\n\nSynonyms:\ncategory_theory/monad.Writer\ncategory_theory/monad Writer","url":"category_theory/monad.html#Writer"},{"doc":"category_theory/monad","title":"fish","content":"pub fn fish(\n  m1: fn(a) -&gt; Writer(b),\n  m2: fn(b) -&gt; Writer(c),\n) -&gt; fn(a) -&gt; Writer(c)\n `Composition` for the embellished functions that return the Writer type.\n \n (&gt;=&gt;) :: (a -&gt; Writer b) -&gt; (b -&gt; Writer c) -&gt; (a -&gt; Writer c)\n m1 &gt;=&gt; m2 = \\x -&gt; \n   let (y, s1) = m1 x\n       (z, s2) = m2 y\n   in (z, s1 ++ s2)\n \n ### Examples\n \n let up_case = fn(s: String) { Writer(string.uppercase(s), &quot;upCase &quot;) }\n let to_words = fn(s: String) { Writer(string.split(s, &quot; &quot;), &quot;toWords &quot;) }\n let process = fish(up_case, to_words)\n process(&quot;Anna has apples&quot;)\n // -&gt; Writer([&quot;ANNA&quot;, &quot;HAS&quot;, &quot;APPLES&quot;], &quot;upCase toWords &quot;)\n \nSynonyms:\ncategory_theory/monad.fish\ncategory_theory/monad fish","url":"category_theory/monad.html#fish"},{"doc":"category_theory/monad","title":"return","content":"pub fn return(x: a) -&gt; Writer(a)\n The `identity morphism` for the Writer category.\n ### Examples\n \n return(2)\n // -&gt; Writer(2, &quot;&quot;)\n return(&quot;abcd&quot;)\n // -&gt; Writer(&quot;abcd&quot;, &quot;&quot;) \n \nSynonyms:\ncategory_theory/monad.return\ncategory_theory/monad return","url":"category_theory/monad.html#return"},{"doc":"category_theory/monad","title":"category_theory/monad","content":" This module contains monad definitions.","url":"category_theory/monad.html"},{"doc":"category_theory/monoid","title":"Monoid","content":"pub type Monoid(m) {\n  Monoid(mempty: m, mappend: fn(m, m) -&gt; m)\n}\n A monoid is a `set` with a `binary operation` or a a `single object category` with a `set of morphisms` that follow the rules of composition.\n \n class Monoid m where\n   mempty  :: m\n   mappend :: m -&gt; m -&gt; m\n \n Laws:\n - mappend mempty x = x\n - mappend x mempty = x\n - mappend x (mappend y z) = mappend (mappend x y) z\n - mconcat = foldr mappend mempty\n ### Examples\n \n let int_sum_monoid = Monoid(mempty: 0, mappend: fn(x: Int, y: Int) { x + y })\n int_sum_monoid\n |&gt; mconcat([2, 3, int_sum_monoid.mempty, 4])\n |&gt; int_sum_monoid.mappend(10)\n // -&gt; 19\n let bool_and_monoid = Monoid(mempty: True, mappend: bool.and)\n True\n |&gt; bool_and_monoid.mappend(False)\n |&gt; bool_and_monoid.mappend(bool_and_monoid.mempty)\n // -&gt; False\n \nMonoid(mempty: m, mappend: fn(m, m) -&gt; m)\n\n\nSynonyms:\ncategory_theory/monoid.Monoid\ncategory_theory/monoid Monoid","url":"category_theory/monoid.html#Monoid"},{"doc":"category_theory/monoid","title":"function_monoid","content":"pub fn function_monoid(mono_b: Monoid(a)) -&gt; Monoid(fn(b) -&gt; a)\n Monoid instance for functions where the `result type` must be a Monoid instance.\nSynonyms:\ncategory_theory/monoid.function_monoid\ncategory_theory/monoid function_monoid","url":"category_theory/monoid.html#function_monoid"},{"doc":"category_theory/monoid","title":"list_monoid","content":"pub fn list_monoid() -&gt; Monoid(List(a))\n Returns the `canonical implementation` of the `monoid type for List`.\n ### Examples\n \n [1, 2]\n |&gt; mono_list.mappend([3, 4, 5])\n |&gt; mono_list.mappend(mono_list.mempty)\n |&gt; mono_list.mappend([6])\n // -&gt; [1, 2, 3, 4, 5, 6]\n \nSynonyms:\ncategory_theory/monoid.list_monoid\ncategory_theory/monoid list_monoid","url":"category_theory/monoid.html#list_monoid"},{"doc":"category_theory/monoid","title":"mconcat","content":"pub fn mconcat(mono: Monoid(a), monoid_list: List(a)) -&gt; a\n Fold a `list of monoids` using mappend. \nSynonyms:\ncategory_theory/monoid.mconcat\ncategory_theory/monoid mconcat","url":"category_theory/monoid.html#mconcat"},{"doc":"category_theory/monoid","title":"option_monoid","content":"pub fn option_monoid(mono_a: Monoid(a)) -&gt; Monoid(Option(a))\n Returns the `canonical implementation` of the `monoid type for Option(a)`. \\\n We must have a Monoid(a) type instance.\n ### Examples\n \n let mono_string = Monoid(mempty: &quot;&quot;, mappend: fn(x: String, y: String) -&gt; String { x &lt;&gt; y })\n let mono_maybe = option_monoid(mono_string)\n \n Some(&quot;ab&quot;)\n |&gt; mono_maybe.mappend(Some(&quot;cd&quot;))\n // -&gt; Some(&quot;abcd&quot;)\n mono_maybe.mappend(Some(&quot;abc&quot;), maybe.mempty)\n // -&gt; None\n \nSynonyms:\ncategory_theory/monoid.option_monoid\ncategory_theory/monoid option_monoid","url":"category_theory/monoid.html#option_monoid"},{"doc":"category_theory/monoid","title":"triple_monoid","content":"pub fn triple_monoid(\n  mono_a: Monoid(a),\n  mono_b: Monoid(b),\n  mono_c: Monoid(c),\n) -&gt; Monoid(#(a, b, c))\n Returns the `canonical implementation` of the `monoid type for Triple`. \\\n We must have a Monoid type instance for a, b, and c.\nSynonyms:\ncategory_theory/monoid.triple_monoid\ncategory_theory/monoid triple_monoid","url":"category_theory/monoid.html#triple_monoid"},{"doc":"category_theory/monoid","title":"tuple_monoid","content":"pub fn tuple_monoid(\n  mono_a: Monoid(a),\n  mono_b: Monoid(b),\n) -&gt; Monoid(#(a, b))\n Returns the `canonical implementation` of the `monoid type for Tuple`. \\\n We must have a Monoid(a) and a Monoid(b) type instance.\nSynonyms:\ncategory_theory/monoid.tuple_monoid\ncategory_theory/monoid tuple_monoid","url":"category_theory/monoid.html#tuple_monoid"},{"doc":"category_theory/monoid","title":"unit_monoid","content":"pub fn unit_monoid() -&gt; Monoid(Nil)\n Returns the `canonical implementation` of the `monoid type for Nil` (unit type).\n ### Examples\n \n mono_unit.mappend(mono_unit.mempty, Nil)\n // -&gt; Nil\n \nSynonyms:\ncategory_theory/monoid.unit_monoid\ncategory_theory/monoid unit_monoid","url":"category_theory/monoid.html#unit_monoid"},{"doc":"category_theory/monoid","title":"category_theory/monoid","content":" The Monoid module contains the `Monoid` type with who&#39;s minimal implementations are `mempty` and `mappend`. \\\n We also define the `mconcat` function in terms of mempty and mappend. \\\n Finally, this module contains several instances for the monoid type.","url":"category_theory/monoid.html"}]);